<?php
/**
 * This class represents a record in the galleries database. It also implements
 * somestatic functions for creating, fetching and deleting records.
 */
class DbGallery {
	private $galleryid = 0;
	private $name = '';
	private $date = '';
	private $isprivate = false;
	
	/**
	 * Constructor. Creates a new gallery record object.
	 * @param integer galleryid the unique id of the gallery record
	 * @param string $name the name of the gallery
	 * @param string $date the creation date of the gallery
	 * @param bool $isprivate flag wheteher this is a private gallery
	 */
	public function __construct($galleryid, $name, $date, $isprivate) {
		Logger::write("Entering DbGallery::__construct", 1);
		$this->galleryid = $galleryid;
		$this->name = $name;
		$this->date = $date;
		$this->isprivate = $isprivate;
	}
	
	/**
	 * Get the unique id of this gallery
	 * @return integer the unique id of this gallery
	 */
	public function getGalleryId() {
		Logger::write("Entering DbGallery::getGalleryId", 1);
		return $this->galleryid;
	}
	
	/**
	 * Get the name of this gallery
	 * @return string the name of this gallery
	 */
	public function getName() {
		Logger::write("Entering DbGallery::getName", 1);
		return $this->name;
	}
	
	/**
	 * Get the creation date of this gallery
	 * @return string the creation date of this gallery
	 */
	public function getDate() {
		Logger::write("Entering DbGallery::getDate", 1);
		return $this->date;
	}
	
	/**
	 * Set the date of this gallery. This function updates the information in the
	 * database directly.
	 * @param string $date the new date of the gallery
	 * @throws DbGalleryException if the date is badly formatted
	 * @throws DbException if database problem
	 */
	public function setDate($date) {
		Logger::write("Entering DbGallery::setDate()", 1);
		if($date == $this->date || $date == '')
			return;
		if(!preg_match("/\d\d\d-\d\d-\d\d/", $date))
			throw new DbGalleryException("DbGallery::setDate", "Date must be on the format YYYY-MM-DD");
		$dbconn = Db::getInstance();
		$query = "UPDATE galleries ".
				 		 "SET date = '$date' ".
				 		 "WHERE id = ". $this->galleryid;
		$dbconn->update($query);
		$this->name = $name;
	}
		
	
	/**
	 * Check whether this gallery is private or not
	 * @return bool the privacy status of this gallery
	 */
	public function isPrivate() {
		Logger::write("Entering DbGallery::isPrivate", 1);
		return $this->isprivate;
	}
	
	/**
	 * Set the name of this gallery. This function updates the information in
	 * the database directly.
	 * @param string $name the new name of the gallery
	 * @throws DbGalleryException if the new name already exists
	 * @throws DbException if database problem
	 */
	public function setName($name) {
		Logger::write("Entering DbGallery::setName", 1);
		if($name == $this->name || $name == '')
			return;
		if(DbGallery::nameExist($name))
			throw new DbGalleryException("DbGallery::setName", "Gallery name \"". $name. "\" already exists");
		$dbconn = Db::getInstance();
		$query = "UPDATE galleries ".
				 "SET name = '$name' ".
				 "WHERE id = ". $this->galleryid;
		$dbconn->update($query);
		$this->name = $name;
	}
	
	/**
	 * Set the privacy status of this gallery. This function updates the
	 * information in the database directly.
	 * @param bool $isprivate flag whether this gallery is private or not. true
	 * if it is, false otherwise
	 * @throws DbException if database problem
	 */
	public function setPrivate($isprivate) {
		Logger::write("Entering DbGallery::isPrivate", 1);
		if($isprivate)
			$ispriv = 1;
		else
			$ispriv = 0;
		$dbconn = Db::getInstance();
		$query = "UPDATE galleries ".
				 "SET is_private = $ispriv ".
				 "WHERE id = ". $this->galleryid; 
		$dbconn->update($query);
		$this->isprivate = $isprivate;
	}
	
	//*********************************************************************
	// Static functions
	//*********************************************************************

	/**
	 * Check if a name already exists in the database.
	 * @param string $name the name to check towards the database
	 * @return bool true if the name exists, false otherwise
	 * @throws DbException if database problem
	 */
	public static function nameExist($name) {
		Logger::write("Entering DbGallery::nameExist", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT count(name) FROM galleries WHERE name = '$name'";
		$result = $dbconn->query($query);
		$line = $dbconn->fetchRow($result);
		return((bool) $line[0]);
	}

	/**
	 * Create a new gallery database record
	 * @param string $name the name of the new gallery
	 * @param bool $isprivate flag whether this new gallery should be private or
	 * not
	 * @return DbGallery the newly created gallery database record as an
	 * DbGallery object
	 * @throws DbGalleryException if the new gallery could not be created
	 * @throws DbException if database problem
	 */	
	public static function createDbGallery($name, $isprivate) {
		Logger::write("Entering DbGallery::createDbGallery", 1);
		if(DbGallery::nameExist($name))
			throw new DbGalleryException("DbGallery::createGallery", "Gallery name \"". $name. "\" already exists");
		if($isprivate)
			$ispriv = 1;
		else
			$ispriv = 0;
		$dbconn = Db::getInstance();
		$query = "INSERT INTO galleries (name, date, is_private)".
				 "VALUES ('$name', NOW(), $ispriv)";
		$result = $dbconn->insertQueryLast($query, "galleries");
		$line = $dbconn->fetchAssoc($result);
		if(!line)
			throw new DbGalleryException("DbGallery::createGallery", "Could not get the newly created record from the database");
		$dbGallery = new DbGallery($line["id"], $line["name"], $line["date"], $line["is_private"]);
		return $dbGallery;
	}
	
	/**
	 * Get the gallery record with the specified id
	 * @param integer $id the id of the gallery wanted
	 * @return DbGallery the gallery record with the specified id
	 * @throws DbGalleryException if the gallery does not exist
	 * @throws DbException if database problem
	 */
	public static function getDbGallery($galleryid) {
		Logger::write("Entering DbGallery::getDbGallery", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT * FROM galleries WHERE id = $galleryid";
		$result = $dbconn->query($query);
		$line = $dbconn->fetchAssoc($result);
		if(!$line)
			throw new DbGalleryException("DbGallery::getGallery", "Gallery with id '". $galleryid. "' could not be found");
		$dbGallery = new DbGallery($line["id"], $line["name"], $line["date"], $line["is_private"]);
		return $dbGallery;
	}

	/**
	 * Get all galleries from database
	 * @param bool $getprivate flag whether private galleries should be included
	 * @return array an array with all DbGallery objects
	 * @throws DbException if database problem
	 */	
	public static function getAllDbGalleries($getprivate) {
		Logger::write("Entering DbGallery::getAllDbGalleries", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT * FROM galleries";
		if(!$getprivate) {
			$query .= " WHERE is_private = 0";
		}
		$query .= " ORDER BY date, id";
		$result = $dbconn->query($query);
		$dbgalleries = array();
		while($line = $dbconn->fetchAssoc($result)) {
			$dbgalleries[] = new DbGallery($line["id"], $line["name"], $line["date"], $line["is_private"]);			
		}
		return $dbgalleries;
	}
	
	/**
	 * Remove gallery record from database with the given id. All images
	 * that belong to the given gallery have to be removed before the
	 * gallery is removed. This function will throw a DbGalleryException if
	 * there are images in the database that belong to the given gallery.
	 * @param integer $galleryid the id of the gallery record to remove
	 * @throws DbException if database problem
	 */
	public static function removeDbGallery($galleryid) {
		Logger::write("Entering DbGallery::removeDbGallery", 1);
		$dbconn = Db::getInstance();
		$query = "DELETE FROM galleries WHERE id = $galleryid";
		$dbconn->update($query);
	}
}
?>
