<?php
require_once("Logger.php");
require_once("Db.php");
require_once("exceptions/DbImageException.php");
require_once("exceptions/DbException.php");

/**
 * The DbImage class is a representation of a database record in the images
 * table. The static functions are used to create a new record, retrieve
 * existing record(s) and remove existing records from the database.
 * @author martin
 */
class DbImage {
	private $imageid = 0;
	private $name = "";
	private $galleryid = 0;
	private $date = "";
	private $comment = "";
	private $isprivate = false;
	private $hits = 0;
	private $defaultsize = 50;
	private $filename = '';
	private $rotation = 0;
	
	/**
	 * The constructor.
	 * @param integer $imageid the id of the database record
	 * @param string $name the name of the image
	 * @param integer $galleryid the id of the gallery this image belongs to
	 * @param string $date the date of the creation of the image
	 * @param string $comment the comment to the image
	 * @param bool $isprivate whether this image is private or not
	 * @param integer $hits the number of times this image has been viewed
	 * @param integer $defaultsize the default view size in percent of the
	 * original image
	 * @param string $filename the name of the image file (not the same as
	 * $name)
	 */
	function __construct($imageid, $name, $galleryid, $date, $comment, 
						 $isprivate, $hits, $defaultsize, $filename, $rotation) {
		Logger::write("Entering DbImage::__construct", 1);
		$this->imageid = $imageid;
		$this->name = $name;				 	
		$this->galleryid = $galleryid;			 	
		$this->date = $date;				 	
		$this->comment = $comment;				 	
		$this->isprivate = $isprivate;				 	
		$this->hits = $hits;
		$this->defaultsize = $defaultsize;
		$this->filename = $filename;
		$this->rotation = $rotation;
	}
	
	/**
	 * Get the id of this image record
	 */
	public function getImageId() {
		Logger::write("Entering DbImage::getImageId", 1);
		return $this->imageid;
	}
	
	/**
	 * Get the  name of this image
	 * @return string the name of this image
	 */
	public function getName() {
		Logger::write("Entering DbImage::getName", 1);
		return $this->name;
	}
	
	/**
	 * Get the gallery id of this image
	 * @return integer the gallery id of this image
	 */	
	public function getGalleryId() {
		Logger::write("Entering DbImage::getGalleryId", 1);
		return $this->galleryid;
	}
	
	/**
	 * Set the gallery id of this image. This function will update the database
	 * with the new gallery id instantly.
	 * @param integer $galleryid the new gallery id
	 * @throws DbException if database problem
	 */
	public function setGalleryId($galleryid) {
		Logger::write("Entering DbImage::setGalleryId", 1);
		if($this->galleryid == $galleryid)
			return;
		$dbconn = Db::getInstance();
		$query = "UPDATE images ".
				 "SET gallery_id = $galleryid ".
				 "WHERE id = ". $this->imageid;
		$dbconn->update($query);
		$this->galleryid = $galleryid;
	}
	
	/**
	 * Get the creation date of this image. The creation date is either the date
	 * when the image was created (with digital camera, scanner, etc.) if that
	 * information is available (exif) or the date when it was added to the
	 * database.
	 * @return string the creation date of this image
	 */
	public function getDate() {
		Logger::write("Entering DbImage::getDate", 1);
		return $this->date;
	}
	
	/**
	 * Set a new date to this image. This function will update the database
	 * with the new date instantly. The EXIF date of the image will not be 
	 * changed.
	 * @param string $date the new date of this image
	 * @throws DbException if database problem
	 */
	public function setDate($date) {
		Logger::write("Entering DbImage::setDate", 1);
		if($this->date == $date)
			return;
		if(!preg_match("/\d\d\d-\d\d-\d\d/", $date))
			throw new DbImageException("DbImage::setDate", "Date must be on the format YYYY-MM-DD");
		$dbconn = Db::getInstance();
		$query = "UPDATE images ".
						 "SET date = '$date' ".
						 "WHERE id = ". $this->imageid;
		$dbconn->update($query); 
		$this->date = $date;
	}
		
	
	/**
	 * Get the comment of this image
	 * @return string the comment of this image
	 */
	public function getComment() {
		Logger::write("Entering DbImage::getComment", 1);
		return $this->comment;
	}
	
	/**
	 * Set a new comment of this image.This function will update the database
	 * with the new comment instantly.
	 * @param string $comment the new comment of this image
	 * @throws DbException if database problem
	 */	
	public function setComment($comment) {
		Logger::write("Entering DbImage::setComment", 1);
		if($this->comment == $comment)
			return;
		$dbconn = Db::getInstance();
		$query = "UPDATE images ".
				 "SET comment = '$comment' ".
				 "WHERE id = ". $this->imageid;
		$dbconn->update($query);
		$this->comment = $comment;
	}
	
	/**
	 * Get privacy status for this image
	 * @return bool the privacy status of this image
	 */
	public function isPrivate() {
		Logger::write("Entering DbImage::isPrivate", 1);
		return $this->isprivate;
	}
	
	/**
	 * Set new privacy status of this image. This function updates the database
	 * with the new privacy status instantly.
	 * @param bool $isprivate the new privacy status of this image
	 * @throws DbException if database problem
	 */
	public function setPrivate($isprivate) {
		Logger::write("Entering DbImage::setPrivate", 1);
		if($this->isprivate == $isprivate)
			return;
		$isPrivate = (bool) $isprivate;
		if($isprivate)
			$isPriv = 1;
		else
			$isPriv = 0;
		$dbconn = Db::getInstance();
		$query = "UPDATE images ".
				 "SET is_private = $isPriv ".
				 "WHERE id = ". $this->imageid;
		$dbconn->update($query);
		$this->isprivate = $isPrivate;
	}
	
	/**
	 * Get the number of times this image has been viewed.
	 * @return integer the number of times this image has been viewed
	 */
	public function getHits() {
		Logger::write("Entering DbImage::getHits", 1);
		return $this->hits;
	}
	
	/**
	 * Increment the number of times this image has been viewed by 1 (one). This
	 * function updates the database with the new number instantly.
	 * @throws DbException if database problem
	 */
	public function incHits() {
		Logger::write("Entering DbImage::incHits", 1);
		$dbconn = Db::getInstance();
		$hits = $this->hits + 1;
		$query = "UPDATE images ".
				 "SET hits = $hits ".
				 "WHERE id = ". $this->imageid;
		$dbconn->update($query);
		$this->hits = $hits;
	}		
	
	/**
	 * Get the default view size of the image in percent of the original image.
	 * @return integer the default view size
	 */
	public function getDefaultSize() {
		Logger::write("Entering DbImage::getDefaultSize", 1);
		return $this->defaultsize;
	}
	
	/**
	 * Set the default view size of this image in percent of the original image.
	 * This function updates the database with the new number instantly.
	 * @param integer $defaultsize the new default view size
	 * @throws DbException if database problem
	 */
	public function setDefaultSize($defaultsize) {
		Logger::write("Entering DbImage::setDefaultSize", 1);
		if($this->defaultsize == $defaultsize)
			return;
		$dbconn = Db::getInstance();
		$query = "UPDATE images ".
				 "SET default_size = $defaultsize ".
				 "WHERE id = ". $this->imageid;
		$dbconn->update($query);
		$this->defaultsize = $defaultsize;
	}
	
	/**
	 * Get the filename of this image
	 * @return string the filename of this image
	 */
	public function getFileName() {
		Logger::write("Entering DbImage::getFileName", 1);
		return $this->filename;
	} 
	
	/**
	 * Get the rotation of the image in degrees meassured counter-clockwise
	 * @return integer the rotation of the image
	 */
	public function getRotation() {
		Logger::write("Entering DbImage::getRotation", 1);
		return $this->rotation;
	}
	
	/**
	 * Set the rotation of the image in degrees meassured counter-clockwise
	 * @param integer $rotation the rotation of the image
	 */
	public function setRotation($rotation) {
		Logger::write("Entering DbImage::setRotation", 1);
		if($this->rotation == $rotation)
			return;
		$dbconn = Db::getInstance();
		$query = "UPDATE images ".
				 "SET rotation = $rotation ".
				 "WHERE id = ". $this->imageid;
		$dbconn->update($query);
		$this->rotation = $rotation;
	}
	
	/**
	 * Get the previous dbimage in the database, if any
	 * @return DbImage the image in the database that preceeds this image, null
	 * if this image is the first
	 * @throws DbException if database problem
	 */
	public function getPreviousDbImage($getprivate) {
		Logger::write("Entering DbImage::getPreviousDbImage", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT id ".
				 "FROM images ".
				 "WHERE gallery_id = ".$this->galleryid;
		if(!$getprivate)
			$query .= " AND is_private = 0";
		$query .= " ORDER BY date DESC, id DESC";
		$result = $dbconn->query($query);
		$found = false;
		while($line = $dbconn->fetchAssoc($result)) {
			if($found)
				return DbImage::getDbImage($line["id"]);
			if($line["id"] == $this->imageid)
				$found = true;
		}
		return null;
	}
	
	/**
	 * Get the next image in the database, if any
	 * @return DbImage the image in the database that succeeds this image, null
	 * if this image is the last
	 * @throws DbException if database problem
	 */
	public function getNextDbImage($getprivate) {
		Logger::write("Entering DbImage::getPreviousDbImage", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT id ".
				 "FROM images ".
				 "WHERE gallery_id = ".$this->galleryid;
		if(!$getprivate)
			$query .= " AND is_private = 0";
		$query .= " ORDER BY date ASC, id ASC";
		$result = $dbconn->query($query);
		$found = false;
		while($line = $dbconn->fetchAssoc($result)) {
			if($found)
				return DbImage::getDbImage($line["id"]);
			if($line["id"] == $this->imageid)
				$found = true;
		}
		return null;
	}
		
	// **********************************************************************
	// Static functions 
	// **********************************************************************/
	
	/**
	 * Create a new image record.
	 * @param integer $galleryid the is of the gallery this new image should be
	 * added to
	 * @param string $comment the comment of the new image
	 * @param bool $isprivate the privacy status of the new image
	 * @param integer $defaultsize the default view size in percent of the
	 * original image
	 * @param string $filepath the path to the file that should be added to the
	 * gallery
	 * @return DbImage the new image record created
	 * @throws DbException if database problem
	 */
	public static function createDbImage($name, $galleryid, $date, $comment, 
										 $isprivate, $defaultsize, $filename, $rotation = 0) {
		Logger::write("Entering DbImage::createDbImage", 1);
		$dbconn = Db::getInstance();
		$query = "INSERT INTO images " . 
				 "(name, gallery_id, date, comment, is_private, default_size, filename, rotation) ".
				 "VALUES ('$name', $galleryid, '$date', '$comment', $isprivate, '$defaultsize', '$filename', $rotation)";
		$result = $dbconn->insertQueryLast($query, "images");
		$line = $dbconn->fetchAssoc($result);
		$dbimage = new DbImage($line["id"], 
							   $line["name"], 
							   $line["gallery_id"], 
							   $line["date"], 
							   $line["comment"], 
							   $line["is_private"], 
							   $line["hits"], 
							   $line["default_size"], 
							   $line["filename"], 
							   $line["rotation"]);
		return $dbimage;
	}
	
	/**
	 * Get an image record from the database
	 * @param integer $id the id of the image wanted
	 * @return DbImage the image record
	 * @throws DbImageException if fetching the image fails
	 * @throws DbException if database problem
	 */
	public static function getDbImage($imageid) {
		Logger::write("Entering DbImage::getDbImage", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT * FROM images WHERE id = $imageid";
		$result = $dbconn->query($query);
		$line = $dbconn->fetchAssoc($result);
		if(!$line)
			throw new DbImageException("DbImage::getImage", "No image with id '". $id. "' found in database");
		$dbimage = new DbImage($line["id"], 
							   $line["name"], 
							   $line["gallery_id"], 
							   $line["date"], 
							   $line["comment"], 
							   $line["is_private"], 
							   $line["hits"], 
							   $line["default_size"], 
							   $line["filename"], 
							   $line["rotation"]);
		return $dbimage;
	}
	
	/**
	 * Get all image records from a gallery
	 * @param integer $galleryid the id of the gallery with the images asked for
	 * @param bool $getprivate flag whether private images should be included
	 * @return mixed an array with all DbImage objects
	 * @throws DbException if database problem
	 */
	public static function getAllDbImages($galleryid, $getprivate) {
		Logger::write("Entering DbImage::getAllDbImages($galleryid, $getprivate)", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT * FROM images WHERE gallery_id = $galleryid";
		if(!$getprivate) {
			$query .= " AND is_private = 0";
		}
		$query .= " ORDER BY date, id";
		$result = $dbconn->query($query);
		$dbimages = array();
		while($line = $dbconn->fetchAssoc($result)) {
			$dbimages[] = new DbImage($line["id"], 
									  $line["name"], 
									  $line["gallery_id"], 
									  $line["date"], 
									  $line["comment"], 
									  $line["is_private"], 
									  $line["hits"], 
									  $line["default_size"], 
									  $line["filename"], 
									  $line["rotation"]);
		}
		return $dbimages;
	}
	
	/**
	 * Remove image record from the database with the given id
	 * @param integer $imageid the id of the image to be removed
	 * @throws DbException if database problem
	 */
	public static function removeDbImage($imageid) {
		Logger::write("Entering DbImage::removeDbImage", 1);
		$dbconn = Db::getInstance();
		$query = "DELETE FROM images WHERE id = $imageid";
		$dbconn->update($query);
	}
	
	/**
	 * Remove all images from a gallery. 
	 * @param integer $galleryid the id of the gallery to clense from all images
	 * @throws DbException if database problem
	 */
	public static function removeAllDbImages($galleryid) {
		Logger::write("Entering DbImage::removeAllDbImages", 1);
		$dbconn = Db::getInstance();
		$query = "DELETE FROM images WHERE gallery_id = $galleryid";
		$dbconn->update($query);
	}
	
	/**
	 * Get all images sorted by popularity
	 * @param integer $page which page to get
	 * @param boolean $getprivate whether private images should be included or
	 * not
	 * @return an array of DbImage with the images sorted by popularity
	 * @throws DbException if database problem
	 */
	public static function getToplistDbImages($page, $getprivate) {
		Logger::write("Entering DbImage::getFavouriteDbImages", 1);
		$dbconn = Db::getInstance();
		$offset = ($page - 1) * TOPLIST_NUMBER;
		$query = "SELECT * FROM images ";
		if(!$getprivate) {
			$query .= "WHERE is_private = 0 ";
		}
		$query .= "ORDER BY hits DESC, id ASC LIMIT $offset, ".(TOPLIST_NUMBER + 1);
		$result = $dbconn->query($query);
		$dbimages = array();
		while($line = $dbconn->fetchAssoc($result)) {
			$dbimages[] = new DbImage($line["id"], 
									  $line["name"], 
									  $line["gallery_id"], 
									  $line["date"], 
									  $line["comment"], 
									  $line["is_private"], 
									  $line["hits"], 
									  $line["default_size"], 
									  $line["filename"], 
									  $line["rotation"]);
		}
		return $dbimages;
	}
	
	/**
	 * Get image by position in toplist
	 * @param integer $position the position in the toplist
	 * @return DbImage the dbimage at the given position in toplist
	 * @throws DbException if database problem
	 */
	public static function getToplistDbImage($position, $getprivate) {
		Logger::write("Entering DbImage::getToplistDbImage", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT * FROM images ";
		if(!$getprivate) {
			$query .= "WHERE is_private = 0 ";
		}
		$query .= "ORDER BY hits DESC, id ASC LIMIT ". ($position - 1). ", 1";
		$result = $dbconn->query($query);
		if($dbconn->numRows($result) == 0)
			return false;
		$line = $dbconn->fetchAssoc($result);
		$dbimage = new DbImage($line["id"], 
							   $line["name"], 
							   $line["gallery_id"], 
							   $line["date"], 
							   $line["comment"], 
							   $line["is_private"], 
							   $line["hits"], 
							   $line["default_size"], 
							   $line["filename"], 
							   $line["rotation"]);
		return $dbimage;
	}
	
	/**
	 * Get the total number of images in the database
	 * @param boolean $getprivate flag whether private images 
	 * should be included in the count
	 * @return integer the total number of images in the database
	 * @throws DbException if database problem
	 */
	public static function getDbImageCount($getprivate) {
		Logger::write("Entering DbImage::getDbImageCount", 1);
		$dbconn = Db::getInstance();
		$query = "SELECT COUNT(*) FROM images";
		if(!$getprivate)
			$query .= " WHERE is_private = 0";
		$result = $dbconn->query($query);
		$line = $dbconn->fetchRow($result);
		return $line[0];
	}
} 
?>
