<?php
require_once("exceptions/ImageException.php");

/**
 * GIFImage implements Image and represents a GIF image.
 */
class GIFImage extends Image {

	/**
	 * Constructor for GIF images.
	 * @param string $filepath the absolute path and filename to the new image
	 * @throws ImageException if the image resource could not be created
	 */
	function __construct($filepath) {
		Logger::write("Entering GIFImage::__construct", 1);
		parent::__construct($filepath);
		$this->createImageResource();
	}
	
	/**
	 * Output this image to the browser or to a file
	 * @param string $filepath absolute path and filename if image should be
	 * outputted to a file (optional). Default is output to browser
	 * @param integer $quality the quality of the outputted image if the format
	 * supports it. Otherwise it is ignored (optional). Default value is maximum
	 * quality
	 * @throws ImageException if the image resource could not be created,
	 * outputted or destroyed
	 */
	public function outputImage($filepath = '', $quality = 100) {
		Logger::write("Entering GIFImage::outputImage", 1);
		if(!(bool)$filepath) {
			header("Content-type: image/gif");
			if(!imagegif($this->resource))
				throw new ImageException("GIFImage::outputImage", "Could not output the image");
		} else {
			if(!imagegif($this->resource, $filepath))
				throw new ImageException("GIFImage::outputImage", "Could not output the image to file '".$filepath."'");
		}
	}

	/**
	 * Resize this image 
	 * @param integer $width the new width of the resized image
	 * @param integer $height the new height of the resized image
	 * @throws ImageException if the resized image resource could not be
	 * created, copied or destroyed
	 */
	public function resizeImage($width, $height) {
		Logger::write("Entering GIFImage::resizeImage", 1);
		$x = imagesx($this->resource);
		$y = imagesy($this->resource);
		if(!($image = imagecreatetruecolor($width, $height)))
			throw new ImageException("GIFImage::resizeImage", "Could not create new image resource");
		if(!imagecopyresampled($image, $this->resource, 0, 0, 0, 0, $width, $height, $x, $y))
			throw new ImageException("GIFImage::resizeImage", "Could not copy image");
		if(!imagedestroy($this->resource))		
			throw new ImageException("GIFImage::outputResizedImage", "Could not destroy resized image resource");
		$this->width = $width;
		$this->height = $height;
		$this->resource = $image;
	}

	/**
	 * Create a new image resource of this image if it has previously been
	 * destroyed. If an image resource already exists it will be destroyed and
	 * recreated from file pointed to by this object
	 * @throws ImageException if the mage resource could not be created
	 */
	public function createImageResource() {
		Logger::write("Entering GIFImage::createImageResource", 1);
		if(!function_exists(imagecreatefromgif))
			throw new ImageException("GIFImage::createImageResource", "GIF support is not installed on this server");
		if($this->resource != null)
			$this->destroyImageResource();
		$resource = @imagecreatefromgif($this->filepath);
		if(!$resource)
			throw new ImageException("GIFImage::createImageResource", "Could not create image resource from '".$this->filepath."'");
		$this->resource = $resource;	
	}
}
?>