<?php
require_once("exceptions/ImageException.php");

/**
 * The Image class is representing an image file on the file system. The class
 * is abstract and inherited by each image format (e.g. jpg, gif, png). If a new
 * format is created (i.e. a new implementing child class is written) name the
 * class <FILEEXTENSION>Image and the file implementing the class should be
 * named the same with a appending ".php". Example: A new class is written to
 * support psd files. Then the name of the class should be "PSDImage" and the
 * name of the file should be "PSDImage.php"
 */
abstract class Image {
	
	protected $filepath = '';
	protected $filename = '';

	protected $width = 0; // in pixels
	protected $height = 0; // in pixels
	protected $filesize = 0; // in bytes
	protected $fileextension = '';
	protected $date = '';
	protected $resource = null;

	/**
	 * Constructor. Creates a new Image object. Called by the implementing child
	 * classes. The argument passed must be the absolute path and filename of
	 * the image.
	 * @param string $filepath the absolute path and filename of the image
	 * @throws ImageException if the image type is not supported
	 */
	protected function __construct($filepath) {
		Logger::write("Entering Image::__construct", 1);
		
		/* Set the file path and file name of this image */
		$this->setFilePath($filepath);
		
		$imageInfo = getimagesize($filepath);
		$this->fileextension = Image::imageTypeToExtension($imageInfo[2]);
		
		/* Get date from headers */
		$this->date = date("Y-m-d");
		if($imageInfo[2] == 2) {
			if(function_exists(exif_read_data)) {
				if($exif = @exif_read_data($filepath, 'EXIF', true)) {
					if($rawDate = @$exif['IFD0']['DateTime']) {
						list($year, $month, $day, $tail) = split('[: ]', $rawDate, 4);
						$this->date = "$year-$month-$day";
					} elseif($rawdate = @$exif['EXIF']['DateTimeOriginal']) {
						list($year, $month, $day, $tail) = split('[: ]', $rawDate, 4);
						$this->date = "$year-$month-$day";
					}
				}
			}
		}
	}
	
	/**
	 * Destructor. Destroys the image resource to avoid memory leak
	 * @throws ImageException if destroying the image resource fails
	 */
	function __destruct() {
		Logger::write("Entering Image::__destruct", 1);
		$this->destroyImageResource();
	}	
	
	/**
	 * Get the width of the image in pixels
	 * @return integer the width of the image
	 */
	public function getWidth() {
		Logger::write("Entering Image::getWidth", 1);
		return $this->width;
	}
	
	/**
	 * Get the height of the image in pixels
	 * @return integer the height of the image
	 */
	public function getHeight() {
		Logger::write("Entering Image::getHeight", 1);
		return $this->height;
	}
	
	/**
	 * Get the filename of the image
	 * @return string the filename of the image
	 */
	public function getFileName() {
		Logger::write("Entering Image::getFileName", 1);
		return $this->filename;	
	}
	
	/**
	 * Get the absolute path and filename of the image
	 * @return string the absolute path and filename of the image
	 */
	public function getFilePath() {
		Logger::write("Entering Image::getFilePath", 1);
		return $this->filepath;
	}
	
	/**
	 * Set the file path and file name of this image
	 * @param string $filepath the absolute path and file name of this image
	 */
	public function setFilePath($filepath) {
		Logger::write("Entering Image::setFilePath", 1);
		$this->filepath = $filepath;
		$this->filename = basename($filepath);
		$imageInfo = getimagesize($filepath);
		$this->width = $imageInfo[0];
		$this->height = $imageInfo[1];
		$this->filesize = filesize($filepath);
	}
	
	/** 
	 * Get the file extension of the image, e.g. if it is a JPEG image, "jpg"
	 * will be returned
	 * @return string the file extension of the image
	 */
	public function getFileExtension() {
		Logger::write("Entering Image::getFileExtension", 1);
		return $this->fileextension;
	}
	
	/**
	 * Get the size of the image in bytes
	 * @return integer the size of the image
	 */
	public function getImageSize() {
		Logger::write("Entering Image::getImageSize", 1);
		return $this->filesize;
	}
	
	/**
	 * Get the creation date of the image. If the information can be fetched
	 * from the image headers (JPEG and TIFF) this date will be returned,
	 * otherwise the date of the creation of this object will be used.
	 * @return string the creation date of the image
	 */
	public function getDate() {
		Logger::write("Entering Image::getDate", 1);
		return $this->date;
	}
	
	/**
	 * Delete the image from the filesystem and its thumbnail if it exists.
	 * @throws FileDeleteException if the file and/or thumbnail cannot be
	 * deleted
	 * @throws ImageException if the file does not exist in the file system or
	 * if the file could not be deleted
	 */
	public function deleteImage() {
		Logger::write("Entering Image::deleteImage", 1);
		if(file_exists($this->filepath)) {
			if(!unlink($this->filepath)) {
				throw new ImageException("Image::deleteImage", "Could not delete image: \"". $this->filepath. "\"");
			}
		} else {
			throw new ImageException("Image::deleteImage", "The file \"". $this->filepath. "\" does not exist");
		}
		/* If this is a gallery image, delete the thumbnail as well */ 
		$thumbPath = THUMBS_PATH. "/". $this->filename;
		if(file_exists($thumbPath)) {
			if(!unlink($thumbPath)) {
				throw new ImageException("Image::deleteImage", "Could not delete thumbnail: \"". $thumbPath. "\"");
			}
		}
	}
	
	/** 
	 * Move this image to a new location. The new location must be an absolute
	 * path and filename.
	 * @param string $newfilepath the destination path and filename
	 * @throws ImageException if the file could not be moved
	 */
	public function moveImage($newfilepath) {
		Logger::write("Entering Image::moveImage", 1);
		/*
		$this->outputImage($newfilepath, SAVE_QUALITY);
		$this->deleteImage($this->filepath);
		*/
		rename($this->filepath, $newfilepath);
		$this->destroyImageResource();
		$this->setFilePath($newfilepath);
		$this->createImageResource();
	}
	
	/**
	 * Create a thumbnail of this image and copy it to given location. The width
	 * and height parameters are the maximum values of the dimensions of the
	 * thumbnail. The thumnail will keep its aspect ratio which means that if
	 * the width is bigger than the height, the width of the thumbnail will be
	 * $width and the height will be proportional to that.
	 * @param integer $width the maximum width of the thumbnail
	 * @param integer $height the maximum height of the thumbnail
	 * @param string $thumbpath the absolute path and filename of the thumbnail
	 * @throws ImageException if the image could not be resized and written to
	 * file.
	 */
	public function createThumbnail($width, $height, $thumbpath) {
		Logger::write("Entering Image::createThumbnail", 1);
		$aspectRatio = $this->height/$this->width;
		if($aspectRatio > 1) {
			$newWidth = round($width/$aspectRatio);
			$newHeight = $height;
		} else {
			$newWidth = $width;
			$newHeight = round($height*$aspectRatio);
		}
		$this->resizeImage($newWidth, $newHeight);
		$this->outputImage($thumbpath, SAVE_QUALITY);
	}
	
	/**
	 * Resize this image. The ratio argument are the size of the outputted image
	 * in percent of the original image.
	 * @param integer $percent the size of the outputted image in percent of the
	 * original image (value between 0 and 100)
	 * @throws ImageException if the resized image could not be outputted to the
	 * browser
	 */
	public function resizeImagePercent($percent) {
		Logger::write("Entering Image::resizeImagePercent", 1);
		$this->resizeImage(round($this->width * $percent / 100), round($this->height * $percent / 100));
	}
	
	/**
	 * Destroy the image resource of this image. This is done to free memory who
	 * otherwise might cause the application to leak memory.
	 * @throws ImageException if the image resource could not be destroyed
	 */
	public function destroyImageResource() {
		Logger::write("Entering Image::destroyImageResource", 1);
		if(is_resource($this->resource)) {
			if(!imagedestroy($this->resource))
				throw new ImageException("Image::destroyImage", "Could not destroy image resource");
		}
		$this->resource = null;
	}
	
	/**
	 * Rotate the image counter-clockwise the given number of degrees. If the
	 * image is not rotated multiples of 90 (e.g. 90,180,270) the extra area
	 * created will be filled with black
	 * @param float $angle the angle to rotate the image
	 */
	public function rotateImage($angle) {
		Logger::write("Entering Image::rotateImage", 1);
		if($angle == 0)
			return;
		$resource = imagerotate($this->resource, $angle, 0);
		if(!is_resource($resource))
			throw new ImageException("Image::rotateImage", "Could not rotate image");
		$this->resource = $resource;
		$this->width = imagesx($this->resource);
		$this->height = imagesy($this->resource);
	}
	
	//************************************************************************
	// Static functions
	//************************************************************************
	
	/**
	 * Return the corresponding file extension of a filetype
	 * @param integer $imagetype the image type retrieved from imagesize()
	 * @param bool $includedot if a dot (".") should be prepended to the
	 * file extension (optional). Default is false
	 * @return string the file extension of the image type
	 * @throws ImageException if the imagetype is not supported
	 */
	public static function imageTypeToExtension($imagetype, $includedot = false) {
		Logger::write("Entering Image::imageTypeToExtension", 1);
		$fileext = $includedot ? '.' : '';
		switch($imagetype) {
			case IMAGETYPE_GIF:		return $fileext.'gif';
			case IMAGETYPE_JPEG:	return $fileext.'jpg';
			case IMAGETYPE_PNG:		return $fileext.'png';
			case IMAGETYPE_SWF:		return $fileext.'swf';
			case IMAGETYPE_PSD:		return $fileext.'psd';
			case IMAGETYPE_WBMP:	return $fileext.'wbmp';
			case IMAGETYPE_XBM:		return $fileext.'xbm';
			case IMAGETYPE_TIFF_II:	return $fileext.'tiff';
			case IMAGETYPE_TIFF_MM:	return $fileext.'tiff';
			case IMAGETYPE_IFF:		return $fileext.'aiff';
			case IMAGETYPE_JB2:		return $fileext.'jb2';
			case IMAGETYPE_JPC:		return $fileext.'jpc';
			case IMAGETYPE_JP2:		return $fileext.'jp2';
			case IMAGETYPE_JPX:		return $fileext.'jpf';
			case IMAGETYPE_SWC:		return $fileext.'swc';
			default: 				
				throw new ImageException("Image::imageTypeToExtension", "Image type '".$imagetype."' is not supported");
		}
	}

	/**
	 * Return a new Image object representing the given image. Depending
	 * on the type of the image it will return the corresponding child class of
	 * the Image.
	 * @param string $filepath the absolute path and filename of the image
	 * @return Image an image object of the specified type
	 * @throws ImageException if an unsupported image format is uploaded
	 */
	public static function getImage($filepath) {
		Logger::write("Entering Image::getImage", 1);
		$imageinfo = getimagesize($filepath);
		$fileextension = Image::imageTypeToExtension($imageinfo[2]);
		$classname = strtoupper($fileextension)."Image";
		if(!(include_once $classname.'.php'))
			throw new ImageException("Image::getImage", "Image type \"". $fileextension. "\" is not supported");
		$imageobj = new $classname($filepath);
		if(!is_object($imageobj))
			throw new ImageException("Image::getImage", "Could not create Image object from child class '".$classname."'");
		return $imageobj;
	}
	
	/**
	 * Create a new image in the gallery file system. The given image will be
	 * moved to the gallery file system and a thumbnail will be created.
	 * @param string $filepath the absolute path and filename of the image to be
	 * added to the gallery file system
	 * @param integer $angle angle to rotate the image (optional). Default is 0
	 * degrees
	 * @throws ImageException if the file could not be moved to the gallery file
	 * system, the thumnail could not be created or an unsupported image format
	 * is uploaded
	 */
	
	public static function createImage($filepath, $angle = 0) {
		Logger::write("Entering Image::createImage", 1);
		$imageFile = Image::getImage($filepath);
		$uniqueFileName = uniqid('').".". $imageFile->getFileExtension();
		$fullFilePath = IMAGES_PATH. "/". $uniqueFileName;
		$thumbFilePath = THUMBS_PATH. "/". $uniqueFileName;
		$origFileName = $imageFile->getFileName();
		$imageFile->moveImage($fullFilePath);
		if((bool) ($angle % 360))
			$imageFile->rotateImage($angle);
		$imageFile->createThumbNail(THUMBS_WIDTH, THUMBS_HEIGHT, $thumbFilePath);
		$imageFile->createImageResource();
		return $imageFile;
	}		

	//************************************************************************
	// Abstract functions
	//************************************************************************

	/**
	 * Output this image to the browser or to a file
	 * @param string $filepath absolute path and filename if image should be
	 * outputted to a file (optional). Default is output to browser
	 * @param integer $quality the quality of the outputted image if the format
	 * supports it. Otherwise it is ignored (optional). Default value is maximum
	 * quality
	 * @throws ImageException if the image resource could not be created,
	 * outputted or destroyed
	 */
	abstract function outputImage($filepath = '', $quality = 100);

	/**
	 * Resize the image resource of this image
	 * @param integer $width the width of the resized copy of this image
	 * @param integer $height the height of the resized copy of this image
	 * @throws ImageException if the resized image resource could not be
	 * created, outputted or destroyed
	 */
	abstract function resizeImage($width, $height);

	/**
	 * Create a new image resource of this image if it has previously been
	 * destroyed. If an image resource already exists it will be destroyed and
	 * recreated from file pointed to by this object
	 * @throws ImageException if the mage resource could not be created
	 */
	abstract function createImageResource();
}
?>
