<?php
require_once("exceptions/ImageException.php");

/**
 * JPGImage implements Image and represents a JPEG image.
 */
class JPGImage extends Image {
	
	/**
	 * Constructor for JPEG images.
	 * @param string $filepath the absolute path and filename to the new image
	 * @throws ImageResource if the image resource could not be created
	 */
	function __construct($filepath) {
		Logger::write("Entering JPGImage::_construct", 1);
		parent::__construct($filepath);
		$this->createImageResource();
	}
	
	/**
	 * Output this image to the browser or to a file
	 * @param string $filepath absolute path and filename if image should be
	 * outputted to a file (optional). Default is output to browser
	 * @param integer $quality the quality of the outputted image if the format
	 * supports it. Otherwise it is ignored (optional). Default value is maximum
	 * quality
	 * @throws ImageException if the image resource could not be created,
	 * outputted or destroyed
	 */
	public function outputImage($filepath = '', $quality = 100) {
		Logger::write("Entering JPGImage::outputImage", 1);
		if(!(bool)$filepath)
			header("Content-type: image/jpeg");
		if(!imagejpeg($this->resource, $filepath, $quality)) {
			throw new ImageException("JPGImage::outputImage", "Could not output the image");
		}
	}

	/**
	 * Resize the image resource of this image
	 * @param integer $width the width of the resized copy of this image
	 * @param integer $height the height of the resized copy of this image
	 * @throws ImageException if the resized image resource could not be
	 * created, outputted or destroyed
	 */
	public function resizeImage($width, $height) {
		Logger::write("Entering JPGImage::resizeImage", 1);
		if(!($image = imagecreatetruecolor($width, $height)))
			throw new ImageException("JPGImage::resizeImage", "Could not create new image resource");
		if(!imagecopyresampled($image, $this->resource, 0, 0, 0, 0, $width, $height, $this->width, $this->height))
			throw new ImageException("JPGImage::resizeImage", "Could not copy image");
		if(!imagedestroy($this->resource))
			throw new ImageException("JPGImage::resizeImage", "Could not destroy original image resource");
		$this->width = $width;
		$this->height = $height;
		$this->resource = $image;
	}

	/**
	 * Create a new image resource of this image if it has previously been
	 * destroyed. If an image resource already exists it will be destroyed and
	 * recreated from file pointed to by this object
	 * @throws ImageException if the mage resource could not be created
	 */
	public function createImageResource() {
		Logger::write("Entering JPGImage::createImageResource", 1);
		if(!function_exists(imagecreatefromjpeg))
			throw new ImageException("JPGImage::createImageResource", "JPEG support is not installed on this server");
		if($this->resource != null)
			$this->destroyImageResource();
		$this->resource = imagecreatefromjpeg($this->filepath);
		if(!is_resource($this->resource))
			throw new ImageException("JPGImage::createImageResource", "Could not create image resource from '".$this->filepath."'");
	}
}
?>
